#!/usr/bin/env python3

import rclpy
import time
from rclpy.action import ActionClient
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor

from rclpy.callback_groups import MutuallyExclusiveCallbackGroup, ReentrantCallbackGroup
from action_tutorials_interfaces.action import Fibonacci

from std_msgs.msg import String
from std_srvs.srv import SetBool

class Node2(Node):
    def __init__(self):
        super().__init__("the_action_client")
        self._action_client = ActionClient(self, Fibonacci, 'fibonacci')
        self._action_client.wait_for_server()
        self.result_future = None

    def send_goal(self):
        goal_msg = Fibonacci.Goal()
        goal_msg.order = 5


        send_goal_future = self._action_client.send_goal_async(goal_msg)
        print("wait2")
        rclpy.spin_until_future_complete(self, send_goal_future)#, executor=self.executor)
        print("wait3")

        self.goal_handle = send_goal_future.result()
        self.result_future = self.goal_handle.get_result_async()
        print("goal sent.")
        return True

    def cancel_goal(self):
        print("Should cancel goal")
        if self.result_future:
            future = self.goal_handle.cancel_goal_async()
            print("wait1")
            rclpy.spin_until_future_complete(self, future)
        pass


class MyActionClient(Node):

    def __init__(self, executor):
        super().__init__('fibonacci_action_client')
        cb = ReentrantCallbackGroup
        self.executor = executor
        self._acClient = Node2()

        self.create_subscription(String, "/data", self._onData, 10)
        self.create_subscription(String, "/data2", self._onData2, 10)
        self.create_service(SetBool, "/mode", self._onMode, callback_group=cb())

        print("Client running...")

        self._callAct = False


    def _onData(self, msg):
        if self._callAct:
            self._acClient.send_goal()

    def _onData2(self, msg):
        self.send_goal()


    def _onMode(self, req, res):
        print("On Mode srv: ", req.data)
        if req.data:
            self._callAct = True
        else:
            print("Cancelling task")
            self._callAct = False
            self._acClient.cancel_goal()

        res.success = True
        print("Return on mode")
        return res


def main(args=None):
    rclpy.init(args=args)

    e = MultiThreadedExecutor(num_threads=8)
    action_client = MyActionClient(e)

    #e.add_node(action_client)
    rclpy.spin(action_client, e)
    #e.spin()



if __name__ == '__main__':
    main()
