#!/usr/bin/env python3

import time

import rclpy
from rclpy.action import ActionServer, GoalResponse
from rclpy.node import Node
from rclpy.executors import MultiThreadedExecutor
from rclpy.callback_groups import MutuallyExclusiveCallbackGroup, ReentrantCallbackGroup

from action_tutorials_interfaces.action import Fibonacci


class MyActionServer(Node):

    def __init__(self):
        super().__init__('fibonacci_action_server')
        self._action_server = ActionServer(
            self,
            Fibonacci,
            'fibonacci',
            self.execute_callback,
            goal_callback=self.goal_callback,
            callback_group = ReentrantCallbackGroup())

    def goal_callback(self, goal_handle):
        print("goal callback")
        return GoalResponse.ACCEPT

    def execute_callback(self, goal_handle):
        print("action1")
        self.get_logger().info('Executing goal...')
        time.sleep(0.1)
        goal_handle.succeed()
        result = Fibonacci.Result()
        result.sequence = [1, 2, 3, 4]
        return result


def main(args=None):
    rclpy.init(args=args)

    fibonacci_action_server = MyActionServer()
    e = MultiThreadedExecutor(num_threads=8)
    e.add_node(fibonacci_action_server)
    e.spin()



if __name__ == '__main__':
    main()
